function encode(p) {
    return encodeURIComponent(p);
}

function serialize(p) {
    switch (typeof p) {
        case 'boolean':
            return p ? 'true' : 'false';
        case 'number':
            return isFinite(p) ? p : '';
        case 'string':
            return p;
        default:
            return '';
    }
}

function serializeQuery(query) {
    return Object.keys(query).map(function (key) {
        var k = encode(key) + '=';

        if (Array.isArray(query[key])) {
            return query[key].map(function (v) {
                return k + encode(serialize(v));
            }).join('&');
        }

        return k + encode(serialize(query[key]));
    }).join('&');
}

function isObject(v) {
    return typeof v === 'object' && v !== null;
}

function isString(v) {
    return typeof v === 'string';
}

function isUndefined(v) {
    return v === undefined;
}

var baseUrl = 'http://localhost/';

function normalizeUrl(url) {
    if (url instanceof URL) {
        return url;
    }

    if (isObject(url)) {
        var urlInstance = new URL('', baseUrl);

        if (url.href) {
            return urlInstance.href = url.href;
        }

        urlInstance.protocol = url.protocol;

        if (url.host) {
            urlInstance.host = url.host;
        } else if (url.hostname || url.port) {
            urlInstance.hostname = url.hostname;
            urlInstance.port = url.port;
        }

        if (url.pathname) {
            urlInstance.pathname = url.pathname;
        }

        if (url.search) {
            urlInstance.search = url.search;
        } else if (url.query) {
            var query = url.query;
            var search = isObject(query) ? serializeQuery(query) : query;
            urlInstance.search = search || '';
        }

        if (url.hash) {
            urlInstance.hash = url.hash;
        }

        if (url.auth) {
            var parts = url.auth.split(':');
            urlInstance.username = parts[0] || '';
            urlInstance.password = parts[1] || '';
        }

        return urlInstance;
    }

    if (isString(url) && url.length > 0) {
        // treat "abc" as "http://abc/" and "/abc" as "http://localhost/abc"
        var urlString = /^(file|ftp|gopher|https?):\/\/|^\//.test(url) ? url : '//' + url;
        return new URL(urlString, baseUrl);
    }

    return new URL('', baseUrl);
}

function escapeQuote(s) {
    return s.replace(/"/g, '\"');
}

function formatCurl(url, options) {
    var _ref = options || {},
        args = _ref.args,
        body = _ref.body,
        headers = _ref.headers,
        json = _ref.json,
        method = _ref.method,
        query = _ref.query;

    var urlInstance = normalizeUrl(url);

    if (query) {
        var search = isObject(query) ? serializeQuery(query) : query;
        urlInstance.search = search || '';
    }

    var result = 'curl "' + urlInstance.toString() + '"';

    var computedHeaders = Object.assign({}, headers);

    if (json) {
        // header names are case insensetive, so we have to normalize them first
        // in order to check for their existance.
        var headersMap = Object.keys(computedHeaders).reduce(function (m, headerName) {
            m[headerName.toLowerCase()] = headerName;
            return m;
        }, {});

        var accept = headersMap.accept || 'accept';
        var contentType = headersMap['content-type'] || 'content-type';

        if (isUndefined(computedHeaders[accept])) {
            computedHeaders[accept] = 'application/json';
        }

        if (isUndefined(computedHeaders[contentType])) {
            computedHeaders[contentType] = 'application/json';
        }
    }

    var headersNames = Object.keys(computedHeaders);

    if (headersNames.length > 0) {
        var headerString = headersNames.map(function (k) {
            var v = isString(computedHeaders[k]) ? escapeQuote(computedHeaders[k]) : computedHeaders[k];

            return '-H "' + k + ': ' + v + '"';
        }).join(' ');

        if (headerString) {
            result += ' ' + headerString;
        }
    }

    if (body) {
        var bodyString = isString(body) ? body : JSON.stringify(body);
        result += ' --data \'' + escapeQuote(bodyString) + '\'';
    }

    if (isString(method)) {
        result += ' -X ' + method.toUpperCase();
    }

    if (Array.isArray(args) && args.length > 0) {
        result = result + ' ' + args.join(' ');
    }

    return result;
}

export default formatCurl;
